#!/usr/bin/python

# Shift register pins
DATAIN=0 #DS
LATCH=1  #STCP
CLOCK=18  #SHCP

# LEDs
led1=0x80 #10000000
led2=0x40 #01000000
led3=0x20 #00100000
led4=0x10 #00010000
led5=0x08 #00001000
led6=0x04 #00000100
led7=0x02 #00000010
led8=0x01 #00000001

#
# based on code from lrvick and LiquidCrystal
# lrvic - https://github.com/lrvick/raspi-hd44780/blob/master/hd44780.py
# LiquidCrystal - https://github.com/arduino/Arduino/blob/master/libraries/LiquidCrystal/LiquidCrystal.cpp
#

from time import sleep

class Adafruit_CharLCD:

    # commands
    LCD_CLEARDISPLAY 		= 0x01
    LCD_RETURNHOME 		= 0x02
    LCD_ENTRYMODESET 		= 0x04
    LCD_DISPLAYCONTROL 		= 0x08
    LCD_CURSORSHIFT 		= 0x10
    LCD_FUNCTIONSET 		= 0x20
    LCD_SETCGRAMADDR 		= 0x40
    LCD_SETDDRAMADDR 		= 0x80

    # flags for display entry mode
    LCD_ENTRYRIGHT 		= 0x00
    LCD_ENTRYLEFT 		= 0x02
    LCD_ENTRYSHIFTINCREMENT 	= 0x01
    LCD_ENTRYSHIFTDECREMENT 	= 0x00

    # flags for display on/off control
    LCD_DISPLAYON 		= 0x04
    LCD_DISPLAYOFF 		= 0x00
    LCD_CURSORON 		= 0x02
    LCD_CURSOROFF 		= 0x00
    LCD_BLINKON 		= 0x01
    LCD_BLINKOFF 		= 0x00

    # flags for display/cursor shift
    LCD_DISPLAYMOVE 		= 0x08
    LCD_CURSORMOVE 		= 0x00

    # flags for display/cursor shift
    LCD_DISPLAYMOVE 		= 0x08
    LCD_CURSORMOVE 		= 0x00
    LCD_MOVERIGHT 		= 0x04
    LCD_MOVELEFT 		= 0x00

    # flags for function set
    LCD_8BITMODE 		= 0x10
    LCD_4BITMODE 		= 0x00
    LCD_2LINE 			= 0x08
    LCD_1LINE 			= 0x00
    LCD_5x10DOTS 		= 0x04
    LCD_5x8DOTS 		= 0x00



    def __init__(self, pin_rs=25, pin_e=24, pins_db=[23, 17, 21, 22], GPIO = None):
	# Emulate the old behavior of using RPi.GPIO if we haven't been given
	# an explicit GPIO interface to use
	if not GPIO:
	    import RPi.GPIO as GPIO
   	self.GPIO = GPIO
        self.pin_rs = pin_rs
        self.pin_e = pin_e
        self.pins_db = pins_db

        self.GPIO.setmode(GPIO.BCM)
        self.GPIO.setup(self.pin_e, GPIO.OUT)
        self.GPIO.setup(self.pin_rs, GPIO.OUT)

        for pin in self.pins_db:
            self.GPIO.setup(pin, GPIO.OUT)

	self.write4bits(0x33) # initialization
	self.write4bits(0x32) # initialization
	self.write4bits(0x28) # 2 line 5x7 matrix
	self.write4bits(0x0C) # turn cursor off 0x0E to enable cursor
	self.write4bits(0x06) # shift cursor right

	self.displaycontrol = self.LCD_DISPLAYON | self.LCD_CURSOROFF | self.LCD_BLINKOFF

	self.displayfunction = self.LCD_4BITMODE | self.LCD_1LINE | self.LCD_5x8DOTS
	self.displayfunction |= self.LCD_2LINE

	""" Initialize to default text direction (for romance languages) """
	self.displaymode =  self.LCD_ENTRYLEFT | self.LCD_ENTRYSHIFTDECREMENT
	self.write4bits(self.LCD_ENTRYMODESET | self.displaymode) #  set the entry mode

        self.clear()
	
	# Shift register setup
	GPIO.setup(DATAIN,GPIO.OUT)
        GPIO.setup(CLOCK,GPIO.OUT)
        GPIO.setup(LATCH,GPIO.OUT)

        GPIO.output(LATCH,False)
        GPIO.output(CLOCK,False)
        GPIO.output(DATAIN,False)

    def begin(self, cols, lines):

	if (lines > 1):
		self.numlines = lines
    		self.displayfunction |= self.LCD_2LINE
		self.currline = 0


    def home(self):

	self.write4bits(self.LCD_RETURNHOME) # set cursor position to zero
	self.delayMicroseconds(3000) # this command takes a long time!
	

    def clear(self):

	self.write4bits(self.LCD_CLEARDISPLAY) # command to clear display
	self.delayMicroseconds(3000)	# 3000 microsecond sleep, clearing the display takes a long time


    def setCursor(self, col, row):

	self.row_offsets = [ 0x00, 0x40, 0x14, 0x54 ]

	if ( row > self.numlines ): 
		row = self.numlines - 1 # we count rows starting w/0

	self.write4bits(self.LCD_SETDDRAMADDR | (col + self.row_offsets[row]))


    def noDisplay(self): 
	""" Turn the display off (quickly) """

	self.displaycontrol &= ~self.LCD_DISPLAYON
	self.write4bits(self.LCD_DISPLAYCONTROL | self.displaycontrol)


    def display(self):
	""" Turn the display on (quickly) """

	self.displaycontrol |= self.LCD_DISPLAYON
	self.write4bits(self.LCD_DISPLAYCONTROL | self.displaycontrol)


    def noCursor(self):
	""" Turns the underline cursor on/off """

	self.displaycontrol &= ~self.LCD_CURSORON
	self.write4bits(self.LCD_DISPLAYCONTROL | self.displaycontrol)


    def cursor(self):
	""" Cursor On """

	self.displaycontrol |= self.LCD_CURSORON
	self.write4bits(self.LCD_DISPLAYCONTROL | self.displaycontrol)


    def noBlink(self):
	""" Turn on and off the blinking cursor """

	self.displaycontrol &= ~self.LCD_BLINKON
	self.write4bits(self.LCD_DISPLAYCONTROL | self.displaycontrol)


    def noBlink(self):
	""" Turn on and off the blinking cursor """

	self.displaycontrol &= ~self.LCD_BLINKON
	self.write4bits(self.LCD_DISPLAYCONTROL | self.displaycontrol)


    def DisplayLeft(self):
	""" These commands scroll the display without changing the RAM """

	self.write4bits(self.LCD_CURSORSHIFT | self.LCD_DISPLAYMOVE | self.LCD_MOVELEFT)


    def scrollDisplayRight(self):
	""" These commands scroll the display without changing the RAM """

	self.write4bits(self.LCD_CURSORSHIFT | self.LCD_DISPLAYMOVE | self.LCD_MOVERIGHT);


    def leftToRight(self):
	""" This is for text that flows Left to Right """

	self.displaymode |= self.LCD_ENTRYLEFT
	self.write4bits(self.LCD_ENTRYMODESET | self.displaymode);


    def rightToLeft(self):
	""" This is for text that flows Right to Left """
	self.displaymode &= ~self.LCD_ENTRYLEFT
	self.write4bits(self.LCD_ENTRYMODESET | self.displaymode)


    def autoscroll(self):
	""" This will 'right justify' text from the cursor """

	self.displaymode |= self.LCD_ENTRYSHIFTINCREMENT
	self.write4bits(self.LCD_ENTRYMODESET | self.displaymode)


    def noAutoscroll(self): 
	""" This will 'left justify' text from the cursor """

	self.displaymode &= ~self.LCD_ENTRYSHIFTINCREMENT
	self.write4bits(self.LCD_ENTRYMODESET | self.displaymode)


    def write4bits(self, bits, char_mode=False):
        """ Send command to LCD """

	self.delayMicroseconds(1000) # 1000 microsecond sleep

        bits=bin(bits)[2:].zfill(8)

        self.GPIO.output(self.pin_rs, char_mode)

        for pin in self.pins_db:
            self.GPIO.output(pin, False)

        for i in range(4):
            if bits[i] == "1":
                self.GPIO.output(self.pins_db[::-1][i], True)

	self.pulseEnable()

        for pin in self.pins_db:
            self.GPIO.output(pin, False)

        for i in range(4,8):
            if bits[i] == "1":
                self.GPIO.output(self.pins_db[::-1][i-4], True)

	self.pulseEnable()


    def delayMicroseconds(self, microseconds):
	seconds = microseconds / float(1000000)	# divide microseconds by 1 million for seconds
	sleep(seconds)


    def pulseEnable(self):
	self.GPIO.output(self.pin_e, False)
	self.delayMicroseconds(1)		# 1 microsecond pause - enable pulse must be > 450ns 
	self.GPIO.output(self.pin_e, True)
	self.delayMicroseconds(1)		# 1 microsecond pause - enable pulse must be > 450ns 
	self.GPIO.output(self.pin_e, False)
	self.delayMicroseconds(1)		# commands need > 37us to settle


    def message(self, text):
        """ Send string to LCD. Newline wraps to second line"""

        for char in text:
            if char == '\n':
                self.write4bits(0xC0) # next line
            else:
                self.write4bits(ord(char),True)


if __name__ == '__main__':

    lcd = Adafruit_CharLCD()
    lcd.clear()

import RPi.GPIO as GPIO
import subprocess
import time

lcd.message("NAS4Pi v1.0\nBooting...")
time.sleep(5)

# Check Disks at start up

hdds = ['/dev/sda', '/dev/sdb', '/dev/sdc', '/dev/sdd']
#hdds = ['/dev/sda']

for hdd in hdds:
	checkdisk1 = subprocess.check_output("/usr/sbin/smartctl -a "+hdd+" | grep Reallocated_Sector_Ct | awk 'NR==1 {print $10}'", shell=True)
	print hdd+" - Reallocated_Sector_Ct is", checkdisk1[:1]
	checkdisk1int = int(checkdisk1)

	checkdisk2 = subprocess.check_output("/usr/sbin/smartctl -a "+hdd+" | grep Current_Pending_Sector | awk 'NR==1 {print $10}'", shell=True)
	print hdd+" - Current_Pending_Sector is", checkdisk2[:-1]
	checkdisk2int = int(checkdisk2)

	checkdisk3 = subprocess.check_output("/usr/sbin/smartctl -a "+hdd+" | grep Offline_Uncorrectable | awk 'NR==1 {print $10}'", shell=True)
	print hdd+" - Offline_Uncorrectable is", checkdisk3[:-1]
	checkdisk3int = int(checkdisk3)

	lcd.clear()
	lcd.message("Check "+hdd)
	if (checkdisk1int == 0 and checkdisk2int == 0 and checkdisk3int == 0):
        	print "- Passed"
        	lcd.message("\nPassed")
	else:
        	print "- Failing"
        	lcd.message("\n*** Failing ***")
		time.sleep(1)
		
		lcd.clear()
		lcd.message("Check "+hdd)
		lcd.message("\nRealloc: ")
	        lcd.message(checkdisk1[:-1])
		time.sleep(1)
		
		lcd.clear()
	        lcd.message("Check "+hdd)
        	lcd.message("\nCurPend: ")
		lcd.message(checkdisk2[:-1])
		time.sleep(1)
		
		lcd.clear()
	        lcd.message("Check "+hdd)
        	lcd.message("\nOffUncorr: ")
        	lcd.message(checkdisk3[:-1])
        	time.sleep(1)
	time.sleep(1)

# Assemble the array at boot
lcd.clear()
print "Assembling Array"
lcd.message("Assembling Array")
subprocess.call("sudo mdadm --assemble --scan", shell=True)
time.sleep(1)

# Mount the array
lcd.clear()
print "Mounting Array"
lcd.message("Mounting Array")
subprocess.call("sudo mount /dev/md0 /media/raid", shell=True)
time.sleep(1)

# Chmod 777 the mount point
subprocess.call("sudo chmod 777 /media/raid", shell=True)

while (1):
	lcd.clear()
	
	# Check array status
	arraystatus = subprocess.check_output("grep 'level 5' /proc/mdstat | awk 'NR==1 {print $11}' | cut -d '/' -f 2 | cut -d ']' -f 1", shell=True)
	print "Disks in array: "+arraystatus[:-1]
	
	resyncstatus = " "
	try:
		resyncstatus = subprocess.check_output("grep 'resync' /proc/mdstat", shell=True)
		print "Resync status: "+resyncstatus[:-1]
	except Exception: 
  		pass
	
	if (arraystatus.find("4") == 0 and not(resyncstatus.find("resync") > 0)):
        	print "Array ok"
        	lcd.message("Array ok\n")
	else:
        	print "Array failed"
		print resyncstatus.find("resync")
       	
		
		if (resyncstatus.find("resync") > 0):
			arraybuild = subprocess.check_output("grep 'resync' /proc/mdstat | awk 'NR==1 {print $4}'", shell=True)
                        print "Resync: "+arraybuild[:-1]
                        lcd.message("Resync: "+arraybuild[:-1]+"\n")
		else:
			arraybuild = subprocess.check_output("grep 'recovery' /proc/mdstat | awk 'NR==1 {print $4}'", shell=True)
			print "Rebuilding: "+arraybuild[:-1]
	        	lcd.message("Rebuild: "+arraybuild[:-1]+"\n")

	# Drive used / free space
	driveused = subprocess.check_output("df -h | grep /dev/md0 | awk 'NR==1 {print $3}'", shell=True)
	drivetotal = subprocess.check_output("df -h | grep /dev/md0 | awk 'NR==1 {print $2}'", shell=True)
	print driveused[:-1]+"/"+drivetotal[:-1]+" Used"
	lcd.message(driveused[:-1]+"/"+drivetotal[:-1]+" Used")

	# LEDs for SMART drive status
	ledsdisplay = 255
        hdds = ['/dev/sda', '/dev/sdb', '/dev/sdc', '/dev/sdd']

        for hdd in hdds:
                checkdisk1 = subprocess.check_output("/usr/sbin/smartctl -a "+hdd+" | grep Reallocated_Sector_Ct | awk 'NR==1 {print $10}'", shell=True)
                print hdd+" - Reallocated_Sector_Ct is", checkdisk1[:1]
                checkdisk1int = int(checkdisk1)

                checkdisk2 = subprocess.check_output("/usr/sbin/smartctl -a "+hdd+" | grep Current_Pending_Sector | awk 'NR==1 {print $10}'", shell=True)
                print hdd+" - Current_Pending_Sector is", checkdisk2[:-1]
                checkdisk2int = int(checkdisk2)

                checkdisk3 = subprocess.check_output("/usr/sbin/smartctl -a "+hdd+" | grep Offline_Uncorrectable | awk 'NR==1 {print $10}'", shell=True)
                print hdd+" - Offline_Uncorrectable is", checkdisk3[:-1]
                checkdisk3int = int(checkdisk3)

                if (checkdisk1int == 0 and checkdisk2int == 0 and checkdisk3int == 0):
                        print "- Passed"
                        if (hdd == "/dev/sda"):
                                ledsdisplay &= ~led1
                        if (hdd == "/dev/sdb"):
                                ledsdisplay &= ~led3
                        if (hdd == "/dev/sdc"):
                                ledsdisplay &= ~led4
                        if (hdd == "/dev/sdd"):
                                ledsdisplay &= ~led5
                else:
			print "- Failing"

	GPIO.output(LATCH,GPIO.LOW)

        for x in range(0,8):
		print "bit %d = %d" % (x,((ledsdisplay >> x) & 1))
                GPIO.output(DATAIN, ((ledsdisplay >> x) & 1))
                GPIO.output(CLOCK,GPIO.HIGH)
                GPIO.output(CLOCK,GPIO.LOW)
                GPIO.output(DATAIN,GPIO.LOW)

        GPIO.output(LATCH,GPIO.HIGH)

	# Sleep
	time.sleep(60)
